#!/bin/sh

PROGNAME=`basename $0`
VERSION="Version 1.1,"
AUTHOR="2013, Patrick Zambelli (http://www.neteye-blog.com/)"

EXIT_CODE=3
CPU_CORE_WARN=60
CPU_CORE_CRIT=80

print_help() {
   echo "$PROGNAME $VERSION"
   echo "$AUTHOR"
   echo ""
   echo "Usage:"
   echo "--help|-h"
   echo "--version|-v"
   echo "--cpu_nice|-n        Include nice processes"
   echo "--cpu_interrupts|-i  Include cpu hardware and software interrupts"
   echo "--cpu_wait_avg|-a    Include waiting for IO and Stealth Time"
   echo "--warning|-w         Warning on single core CPU usage"
   echo "--critical|-c        Critical on single core CPU usage"
   echo ""
   echo "CPU values from top:"
   echo "- us = userspace"
   echo "- sy = system/kernel"
   echo "- ni = nice processs"
   echo "- id = idle"
   echo "- wa = wiating for IO"
   echo "- hi = hardware interrupts"
   echo "- si = software interrupts"
   echo "- st = Stealth Time"
   echo ""
   echo "In order to be able to run this plugin, you need to:"
   echo "1. Symlink your 'top' executable to /var/tmp/top_toggle_1 (once)"
   echo "2. For every user who will run the plugin:"
   echo "2.a) Run /var/tmp/top_toggle_1 manually"
   echo "2.b) Enter '1', 'W', 'q'"
   exit 3
}

# Check whether we can work at all
if [ ! \( -x /var/tmp/top_toggle_1 -a -f $HOME/.top_toggle_1rc \) ]; then
   echo "UNKNOWN: Cannot run /var/tmp/top_toggle_1"
   echo "In order to be able to run this plugin, you need to:"
   echo "1. Symlink your 'top' executable to /var/tmp/top_toggle_1 (once)"
   echo "2. For every user who will run the plugin:"
   echo "2.a) Run /var/tmp/top_toggle_1 manually"
   echo "2.b) Enter '1', 'W', 'q'"
   exit 3
fi

# parse arguments
while test -n "$1"; do
    case "$1" in
        --help|-h)
            print_help
            exit $EXIT_CODE
            ;;
        --version|-v)
            print_version $PROGNAME $VERSION
            exit $EXIT_CODE
            ;;
        --cpu_nice|-n)
            show_nice=1
	    ;;
        --cpu_interrupts|-i)
            show_interrupts=1
            ;;
        --cpu_wait_avg|-a)
            show_wait=1
            ;;
        --warning|-w)
            CPU_CORE_WARN=$2
            shift
            ;;
        --critical|-c)
            CPU_CORE_CRIT=$2
            shift
            ;;
        *)
            echo "Unknown argument: $1"
            print_help
            exit $EXIT_CODE
            ;;
    esac
    shift
done


# Read CPU usage data. Note that the first line gives the average since
# the last boot, not the current values - hence, we have to wait for the
# 2nd line (average since previous line)!

CPULINES=`/var/tmp/top_toggle_1 -b -n 2 -u nobody | sed -e '1,/PID/d' -e '/^Cpu[0-9]/p' -e d`
CPUNUMS=`echo "$CPULINES" | sed -e 's/^Cpu\([0-9]*\)[^0-9].*/\1/' | tr '\n' ' '`
CPUCOUNT=`echo $CPUNUMS | wc -w`
PERFDATA="cpu_count=$CPUCOUNT"


# Extract the various categories
str_cpu_extract()
{
   PART=`echo "$CPULINE 999" | sed -e 's/% *'"$1"'.*//' -e 's/.*[^0-9\.]//'`
   if [ "$PART" != "999" ]; then
      if [ "$2" = "raw" ]; then
         echo "$PART"
      else
         echo "$1_$CPUNUM=$PART%"
      fi
   fi
}

cpu_extract()
{
   PART=`echo "$CPULINE 999" | sed -e 's/% *'"$1"'.*//' -e 's/.*[^0-9\.]//'`
   if [ "$PART" != "999" ]; then
      if [ "$2" = "raw" ]; then
         echo "$PART"
      else
         echo "$1_$CPUNUM=$PART%;$3;$4;0;${2:-100}"
      fi
   fi
}

for CPUNUM in $CPUNUMS ; do

CPU_CYCLE_STATUS_CODE=3
CPULINE=`echo "$CPULINES" | grep '^Cpu'"$CPUNUM"'[^0-9]'`


PERFDATA="$PERFDATA `cpu_extract us` `cpu_extract sy 100`"
CPUUSER=`cpu_extract us raw`
CPUSYSTEM=`cpu_extract sy raw | sed -e 's/\..*//'`

if [ $show_nice ]; then
   PERFDATA="$PERFDATA `cpu_extract ni $CPUUSER` `cpu_extract id`"
fi

CPUIDLE=`cpu_extract id raw`

if [ $show_wait ]; then
PERFDATA="$PERFDATA `cpu_extract wa`"
fi

if [ $show_interrupts ]; then
PERFDATA="$PERFDATA `cpu_extract hi` `cpu_extract si` `cpu_extract st`"
fi

# If we cannot get the idle percentage, we can just as well give up
if [ "$CPUIDLE" = "" ]; then
   echo "UNKNOWN: Cannot determine idle percentage of CPU $CPUNUM"
   exit 3
fi

CPUTOTAL=`echo $CPUIDLE | LANG=C awk '{ printf "%3.1f",100.0-$1; }'`
PERFDATA="$PERFDATA total_used_$CPUNUM=$CPUTOTAL%;$CPU_CORE_WARN;$CPU_CORE_CRIT;0;100"

# Compare the cpu load of this core and determine return status
CPUTOTAL_INT=`echo $CPUTOTAL | cut -d"." -f1`
if [ "$CPUTOTAL_INT" -gt "$CPU_CORE_CRIT" ];then
   CPU_CYCLE_STATUS_CODE=2
   STR_CPU_USAGE="$STR_CPU_USAGE CRITICAL LOAD"
elif [ "$CPUTOTAL_INT" -gt "$CPU_CORE_WARN" ];then
   CPU_CYCLE_STATUS_CODE=1
   STR_CPU_USAGE="$STR_CPU_USAGE WARNING LOAD"
else
   CPU_CYCLE_STATUS_CODE=0
fi

STR_CPU_USAGE="$STR_CPU_USAGE CPU$CPUNUM: $CPUTOTAL%"
#echo "Core $CPUNUM exit status: $CPU_CYCLE_STATUS_CODE"

# Pass the cycle return value to the global exit status
# Set any bad status
if [ "$CPU_CYCLE_STATUS_CODE" -gt 0 ];then

   # do not overwrite a critical
   if [ "$EXIT_CODE" -ne 2 ];then
      EXIT_CODE=$CPU_CYCLE_STATUS_CODE
   fi
# if still in default unknown
elif [ "$EXIT_CODE" -eq 3 ];then
   EXIT_CODE=$CPU_CYCLE_STATUS_CODE
fi

done

CPUNUMS=`echo "$CPUNUMS" | sed -e 's/^  *//' -e 's/   */ /g' -e 's/  *$//'`
#STR_OUTPUT=`echo "Usage data of $CPUCOUNT CPU cores (numbered $CPUNUMS) $STR_CPU_USAGE |$PERFDATA" | sed -e 's/   */ /'`
STR_OUTPUT=`echo "Usage data of $CPUCOUNT CPU cores $STR_CPU_USAGE |$PERFDATA" | sed -e 's/   */ /'`
if [ "$EXIT_CODE" -eq 2 ];then
   echo "CRITICAL: $STR_OUTPUT"
elif [ "$EXIT_CODE" -eq 1 ];then
   echo "WARNING: $STR_OUTPUT"
elif [ "$EXIT_CODE" -eq 3 ];then
   echo "UNKNOWN: $STR_OUTPUT"
else
   echo "OK: $STR_OUTPUT"
fi

exit $EXIT_CODE
